#include "bezier.h"

Bezier::Bezier(){
    velocity_=11;
    rotation_angle_=0;
    animating_=true;
}

Bezier::~Bezier(){
    glBindVertexArray(vao_);
    glDisableVertexAttribArray(0);
    glBindBuffer(GL_ARRAY_BUFFER, 0);
    glDeleteBuffers(1, &vertices_);
    glBindVertexArray(0);
    glDeleteVertexArrays(1, &vao_);

}

void Bezier::Initialize(const float points [16][4]){

    glGenVertexArrays(1, &vao_);
    glBindVertexArray(vao_);

    glGenBuffers(1, &vertices_);
    glBindBuffer(GL_ARRAY_BUFFER, vertices_);
    glBufferData(GL_ARRAY_BUFFER, 64*sizeof(float), points, GL_STATIC_DRAW);
    glVertexAttribPointer(0, 4, GL_FLOAT, GL_FALSE, 0, nullptr);
    glEnableVertexAttribArray(0);

    glBindVertexArray(0);

}


void Bezier::Move(float dt){
    if(!animating_) return;
    rotation_angle_ += dt* velocity_;
    if (rotation_angle_>360) {rotation_angle_ -=360;}
    model_matrix_.SetUnitMatrix();
    model_matrix_.RotateAboutY(7*rotation_angle_);
    model_matrix_.RotateAboutX(-5*rotation_angle_);
}


void Bezier::Draw(Program &program){
    glBindVertexArray(vao_);
    glUseProgram(program);

    program.SetModelMatrix(model_matrix_);
    program.SetMaterial(material_);

    glEnable(GL_CULL_FACE);
    glCullFace(GL_BACK);

    program.SetLambda(1);
    glFrontFace(GL_CW);
    glPatchParameteri(GL_PATCH_VERTICES, 16);
    glDrawArrays(GL_PATCHES, 0, 16);

    program.SetLambda(-1);
    glFrontFace(GL_CCW);
    glPatchParameteri(GL_PATCH_VERTICES, 16);
    glDrawArrays(GL_PATCHES, 0, 16);

    glDisable(GL_CULL_FACE);
    glBindVertexArray(0);
    glUseProgram(0);
}


