#ifndef PROGRAM_H
#define PROGRAM_H


#include "matma.h"
#include "material.h"
#include "light.h"

class Program{
public:
    ~Program();
    void Initialize(const char * , const char* , const char *, const char *);
    void SetProjectionMatrix(const Mat4 &);
    void SetMaterial(const Material &material);
    void SetLight(const PointLight & light);
    void SetModelMatrix(const Mat4 &);
    void SetViewMatrix(const Mat4 & );
    void SetLambda(GLfloat lambda);

    operator GLuint(){return program_;}

private:

    GLuint program_;

    GLuint vertex_shader_;
    GLuint fragment_shader_;
    GLuint tess_control_shader_;
    GLuint tess_eval_shader_;


    GLuint LoadAndCompileShaderOrDie(const char* source_file, GLenum type);
    GLuint LinkProgramOrDie(GLuint, GLuint, GLuint, GLuint);
    GLint GetUniformLocationOrDie(const char *);

    GLint projection_matrix_location_;
    GLint view_matrix_location_;
    GLint model_matrix_location_;
    GLint lambda_location_;

    struct {
        GLint emission;
        GLint ambient;
        GLint diffuse;
        GLint specular;
        GLint shininess;
    } material_locations_;

    struct {
        GLint ambient;
        GLint attenuation;
        GLint diffuse;
        GLint position;
        GLint specular;
    }light_locations_;

};

#endif // PROGRAM_H
