#ifndef WINDOW_H
#define WINDOW_H


#include <GL/glew.h>
#include <GLFW/glfw3.h>

#include "matma.h"
#include "program.h"
#include "bezier.h"


#include <ctime>


class Window {
public:
    Window(const char*, int, int);
    void Initialize(int major_gl_version, int minor_gl_version);
    void Resize(int new_width, int new_height);
    void KeyEvent(int key, int scancode, int action, int mods);
    void Run(void);
    operator GLFWwindow*(){return window_;}

private:
private:
   int width_;
   int height_;
   const char* title_;
   GLFWwindow* window_;

   Mat4 view_matrix_;
   Mat4 projection_matrix_;
   Program program_;
   Bezier bezier_;
   clock_t last_time_;

   void InitModels();
   void InitPrograms();

   void SetViewMatrix();
   void SetProjectionMatrix();

   void InitGlfwOrDie(int major_gl_version,
                      int minor_gl_version);
   void InitGlewOrDie();

};

#endif // WINDOW_H
